unit Unit1;

{$A-} // Important: Byte aligned structures
      // Do not let the compiler pad the structures for efficiency


// Author: Matt Claessen, Avantes
// Date : 22-05-2008
//
// Minimal Delphi application, for a single spectrometer channel, without using
// AS5216.DLL
// This is the RS232 version of the program. It uses the RS232 library: SuperCom
//
// Software used:
// Borland Delphi, version 6, for native Win32 (as opposed to .Net )
// Steema TeeChart Pro, version 5.02  (to draw the chart)
// Adontec SuperCom, version 4.3
//
// The values for the detector are hardcoded for the Sony ILX 554
// Other detectors have a different number of dead pixels and a different number
// of datapixels
// You will have to change the measurement data structure (xxxxx_measdatatype)
// e.g. The Toshiba TCD-1304 has 13 dead pixels and 3648 datapixels
//      The Hamamatsu 9840 has 8 dead pixels and 1024 datapixels
//
// The low-level protocol used is described in the following Avantes document:
// "AS-5216 RS-232 communication protocol.pdf", available on request.
// This document describes the protocol that is used in both USB and RS-232
// communication. In this demo, only the most important commands are handled.
// Only the structures that are actually used have been changed to include a
// prefix and a postfix.
//
// Use a 3-wire null modem cable (no handshaking lines) with the following
// pinout to connect your AS5216 to a COM port of your PC :
//
//     PC                      AS5216
//  Female DB9 plug         Male HD26 Plug
//
//     2 (TX)  --------------- 19 (RX)
//     3 (RX)  --------------- 10 (TX)
//     5 (GND) ---------------  1 (GND)
//
// If you don't want to wire your own cable, the Avantes order# for this
// cable is IC-DB26/DB9-2
//
// In this demo, all 2048 pixels are used. This number can be reduced by changing
// the measconfig.m_StartPixel and measconfig.m_StopPixel values.
// The number of pixels in the xxxxx_measdatatype has to be reduced accordingly.
// The number of dead pixels is always the same and must not be changed.
// (Unless you change the detectortype, that is)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, TeeProcs, TeEngine, Chart, Series, StdCtrls,
  Buttons, SuperCom;

const STX                     = $02;
      ETX                     = $03;
      DLE                     = $10;
      USER_ID_LEN             = 64;
      NR_WAVELEN_POL_COEF     = 5;
      NR_NONLIN_POL_COEF      = 8;
      NR_DEFECTIVE_PIXELS     = 30;
      MAX_NR_PIXELS           = 4096;
      NR_TEMP_POL_COEF        = 5;
      MAX_TEMP_SENSORS        = 3;
      ROOT_NAME_LEN           = 6;
      AVS_SERIAL_LEN          = 9;
      MAX_PIXEL_VALUE         = $FFFC;
      MAX_VIDEO_CHANNELS      = 2;
      MAX_LASER_WIDTH         = $FFFF;
      HW_TRIGGER_MODE         = 0;
      SW_TRIGGER_MODE         = 1;
      EDGE_TRIGGER_SOURCE     = 0;
      LEVEL_TRIGGER_SOURCE    = 1;
      MAX_TRIGGER_MODE        = 1;
      MAX_TRIGGER_SOURCE      = 1;
      MAX_TRIGGER_SOURCE_TYPE = 1;
      MAX_INTEGRATION_TIME    = 600000;    // 600 seconds
      SAT_DISABLE_DET         = 0;
      SAT_ENABLE_DET          = 1;
      SAT_PEAK_INVERSION      = 2;
      DIG_IN0                 = 0;
      DIG_IN1                 = 1;
      DIG_IN2                 = 2;
      NR_OF_DIGITAL_INPUTS    = 3;
      NR_OF_DIGITAL_OUTPUTS   = 10;
      NR_OF_ANALOG_OUTPUTS    = 2;
      NR_OF_ANALOG_INPUTS     = 2;
      NR_DAC_POL_COEF         = 2;

      ERR_SUCCESS                   = 0;
      ERR_INVALID_PARAMETER         = -1;
      ERR_OPERATION_NOT_SUPPORTED   = -2;
      ERR_DEVICE_NOT_FOUND          = -3;
      ERR_INVALID_DEVICE_ID         = -4;
      ERR_OPERATION_PENDING         = -5;
      ERR_TIMEOUT                   = -6;
      ERR_INVALID_PASSWORD          = -7;
      ERR_INVALID_MEAS_DATA         = -8;
      ERR_INVALID_SIZE              = -9;
      ERR_INVALID_PIXEL_RANGE       = -10;
      ERR_INVALID_INT_TIME          = -11;
      ERR_INVALID_COMBINATION       = -12;
      ERR_INVALID_CONFIGURATION     = -13;
      ERR_NO_MEAS_BUFFER_AVAIL      = -14;
      ERR_UNKNOWN                   = -15;
      ERR_COMMUNICATION             = -16;
      ERR_NO_SPECTRA_IN_RAM         = -17;
      ERR_INVALID_DLL_VERSION       = -18;
      ERR_NO_MEMORY                 = -19;
      ERR_DLL_INITIALISATION        = -20;
      ERR_INVALID_STATE             = -21;

type  int8                          = shortint;
      uint8                         = byte;
      int16                         = smallint;
      uint16                        = word;
      uint32                        = DWORD;
      ptrDWord                      = ^DWORD;

      AvsHandle                     = longint;
      FRNType                       = array[0..ROOT_NAME_LEN-1] of char;

      DEVICE_STATUS                 =  ( UNKNOWN,
                                         AVAILABLE,
                                         IN_USE_BY_APPLICATION,
                                         IN_USE_BY_OTHER);

      SENS_TYPE                     =  ( SENS_HAMS8378_256=1,
                                         SENS_HAMS8378_1024,
                                         SENS_ILX554,
                                         SENS_HAMS9201,
                                         SENS_TCD1304,
                                         SENS_TSL1301,
                                         SENS_TSL1401,
                                         SENS_HAMS8378_512,
                                         SENS_HAMS9840
                                         );

      AvsIdentityType               = record
                                        prefix: array[0..9] of byte;
                                        version1: uint32;
                                        version2: uint32;
                                        version3: uint32;
                                        SerialNumber: array[0..AVS_SERIAL_LEN-1] of char;
                                        UserFriendlyName: array[0..USER_ID_LEN-1] of char;
                                        postfix: array[0..1] of byte;  // DLE/ETX
                                      end;

      ControlSettingsType           = record
                                        m_StrobeControl: uint16;
                                        m_LaserDelay: uint32;
                                        m_LaserWidth: uint32;
                                        m_LaserWaveLength: single;
                                        m_StoreToRam:uint16; 
                                      end;

      DarkCorrectionType            = record
                                        m_Enable: uint8;
                                        m_ForgetPercentage: uint8;
                                      end;

      DetectorType                  = record
                                        m_SensorType: SENS_TYPE;
                                        m_NrPixels: uint16;
                                        m_aFit: array[0..NR_WAVELEN_POL_COEF-1] of single;
                                        m_NLEnable: boolean;
                                        m_aNLCorrect: array[0..NR_NONLIN_POL_COEF-1] of double;
                                        m_aLowNLCounts:double;
                                        m_aHighNLCounts:double;
                                        m_Gain: array[0..MAX_VIDEO_CHANNELS-1] of single;
                                        m_Reserved: single;
                                        m_Offset: array[0..MAX_VIDEO_CHANNELS-1] of single;
                                        m_ExtOffset: single;
                                        m_DefectivePixels: array[0..NR_DEFECTIVE_PIXELS-1] of uint16;
                                      end;

      SmoothingType                 = record
                                        m_SmoothPix: uint16;
                                        m_SmoothModel: uint8;
                                      end;

      SpectrumCalibrationType       = record
                                        m_Smoothing: SmoothingType;
                                        m_CalInttime: single;
                                        m_aCalibConvers: array[0..MAX_NR_PIXELS-1] of single;
                                      end;

      IrradianceType                = record
                                        m_IntensityCalib: SpectrumCalibrationType;
                                        m_CalibrationType: uint8;
                                        m_FiberDiameter: uint32;
                                      end;

      TriggerType                   = record
                                        m_Mode: uint8;
                                        m_Source: uint8;
                                        m_SourceType:uint8;
                                      end;

      MeasConfigType                = record
                                        m_StartPixel: uint16;
                                        m_StopPixel: uint16;
                                        m_IntegrationTime: single;
                                        m_IntegrationDelay: uint32;
                                        m_NrAverages: uint32;
                                        m_CorDynDark: DarkCorrectionType;
                                        m_Smoothing: SmoothingType;
                                        m_SaturationDetection: uint8;
                                        m_Trigger: TriggerType;
                                        m_Control: ControlSettingsType;
                                      end;

      // bugfix 09-09-08: MeasConfigType is nested in StandAloneType
      // -> do not add prefix and postfix, define a new type

      SendMeasConfigType            = record
                                        prefix: array[0..9] of byte;
                                        m_Meas: MeasConfigType;
                                        postfix: array[0..1] of byte;  // DLE/ETX
                                      end;

      TimeStampType                 = record
                                        m_Date: uint16;
                                        m_Time: uint16;
                                      end;

      SDCardType                    = record
                                        m_Enable: boolean;
                                        m_SpectrumType: uint8;
                                        m_aFileRootName: FRNType;
                                        m_TimeStamp: TimeStampType;
                                      end;

      SpectrumCorrectionType        = record
                                        m_aSpectrumCorrect: array[0..MAX_NR_PIXELS-1] of single;
                                      end;

      StandAloneType                = record
                                        m_Enable: boolean;
                                        m_Meas: MeasConfigType;
                                        m_Nmsr: int16;
                                        m_SDCard: SDCardType;
                                      end;

      TempSensorType                = record
                                        m_aFit: array[0..NR_TEMP_POL_COEF-1] of single;
                                      end;

      TecControlType                = record
                                        m_Enable: boolean;
                                        m_Setpoint: single;   // degrees Celsius
                                        m_aFit:array[0..NR_DAC_POL_COEF-1] of single;
                                      end;

      ProcessControlType            = record
                                        AnalogLow: array[0..NR_OF_ANALOG_OUTPUTS-1] of single;
                                        AnalogHigh: array[0..NR_OF_ANALOG_OUTPUTS-1] of single;
                                        DigitalLow:array[0..NR_OF_DIGITAL_OUTPUTS-1] of single;
                                        DigitalHigh:array[0..NR_OF_DIGITAL_OUTPUTS-1] of single;
                                      end;

const SETTINGS_RESERVED_LEN         = (62*1024) - sizeof(uint32) -
                                      (  sizeof(uint16) +   // m_Len
                                         sizeof(uint16) +   // m_ConfigVersion
                                         USER_ID_LEN +
                                         sizeof(DetectorType) +
                                         sizeof(IrradianceType) +
                                         sizeof(SpectrumCalibrationType) +
                                         sizeof(SpectrumCorrectionType) +
                                         sizeof(StandAloneType) +
                                         (sizeof(TempSensorType)*MAX_TEMP_SENSORS) +
                                         sizeof(TecControlType) +
                                         sizeof(ProcessControlType)
                                      );


type  DeviceConfigType          = record
                                    prefix: array[0..9] of byte;
                                    m_Len: uint16;
                                    m_ConfigVersion: uint16;
                                    m_aUserFriendlyId: array[0..USER_ID_LEN-1] of char;
                                    m_Detector: DetectorType;
                                    m_Irradiance: IrradianceType;
                                    m_Reflectance: SpectrumCalibrationType;
                                    m_SpectrumCorrect: SpectrumCorrectionType;
                                    m_StandAlone: StandAloneType;
                                    m_aTemperature: array[0..MAX_TEMP_SENSORS-1] of TempSensorType;
                                    m_TecControl: TecControlType;
                                    m_ProcessControl: ProcessControlType;
                                    m_aReserved: array[0..SETTINGS_RESERVED_LEN-1] of uint8;
                                    postfix: array[0..1] of byte;  // DLE/ETX
                                  end;
       pDeviceConfigType         = ^DeviceConfigType;

       TPString = array [0..255] of char;
       array12 = array[0..11] of byte;
       array14 = array[0..13] of byte;
       array53 = array[0..52] of byte;

  TForm1 = class(TForm)
    Chart1: TChart;
    Series1: TFastLineSeries;
    GroupBox1: TGroupBox;
    EdtIntTime: TEdit;
    EdtAvg: TEdit;
    EdtNrms: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    open_comm_btn: TButton;
    SingleScanBtn: TButton;
    Edit1: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    Edit2: TEdit;
    close_comm_btn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure open_comm_btnClick(Sender: TObject);
    procedure SingleScanBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure close_comm_btnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    fit:array[0..NR_WAVELEN_POL_COEF-1] of single;
    Avg:cardinal;
    IntTime:single;
    Nrms:int16;
    function readinputvalues:boolean;
    function swapsingle(value_in:single):single;
    function swapint32(value_in:integer):integer;
    function getlambda(pixel:integer):single;
    function clearbuffer:integer;
    function sendarray12(outarray:array12):integer;
    function sendarray14(outarray:array14):integer;
    function sendarray53(outarray:array53):integer;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const time_out:Cardinal=10000;
      // milliseconds, you will have to increase this value if your integration
      // time is in this range as well !!
      deadpix=18;
      startpix=0;
      stoppix=2047;
      comport=COM_1;
      PortBaud:longint=115200;
      PortParity:char='N';
      PortDataBits: byte=8;
      PortStopBits: byte=1;
      PortHandShaking: byte=SIGNAL_NONE;

var PortInBuf: array[0..65535] of char;     // global input buffer
    bps,bpe: word;                          // indexes to begin and end of the data
    escape: boolean;                        // used to handle DLE in input buffer

procedure Snooze(ms: Cardinal);
var
  Stop: Cardinal;
begin
  SetTimer(Application.Handle, 1235, ms, nil);
  Stop := GetTickCount + ms;
  repeat
    Application.HandleMessage;
  until Application.Terminated or (Integer(GetTickCount - Stop) >= 0);
  KillTimer(Application.Handle, 1235);
end;

procedure Timed_Message(const Msg:string;DlgType:TMsgDlgType;
                        Buttons:TMsgDlgButtons;X,Y:Integer);
begin
  with CreateMessageDialog(Msg,DlgType,Buttons) do
    try
      if X>=0 then Left:=X;
      if Y>=0 then Top:=Y;
      Show;
      Snooze(2750);
      Hide;
    finally
      Free;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Chart1.Title.Visible:=false;
  Chart1.View3D:=false;
  Chart1.Legend.Visible:=false;
  Chart1.BottomAxis.Title.Caption:='Wavelength [nm]';
  Chart1.LeftAxis.Title.Caption:='Counts';
  EdtIntTime.Text:=format('%5.1f',[5.0]);
  EdtAvg.Text:=format('%2.0d',[1]);
  EdtNrms.Text:=format('%2.0d',[1]);
  SingleScanBtn.enabled:=false;
end;

// If you call _ComRead directly, outside an event handler, you will find that
// you read past the DLE/ETX marker, making it very hard to synchronize your
// calls (How do you flush a buffer that already contains some of your next
// incoming data ?)
// This event handler is called in the background by the interrupt handler
// in SuperCom, for every incoming event (not for every single byte !)
// If you call _ComRead inside an event handler, you can also inspect the
// incoming data, before moving it to your own buffer.

function EventProc(com:byte;SComEvent:DWORD):DWORD;stdcall;
var i,size:DWORD;
    c:char;
begin
  case SComEvent of
    EV_RXCHAR: begin
                 size:=_ComBufCount(comport,DIR_INC);
                 for i:=0 to size do
                   if (_ComRead(comport,c)) then
                   begin
                     if escape then   // escape=true
                     begin
                       case ord(c) of
                         STX,ETX: begin
                                    PortInBuf[bpe]:=chr(DLE);  // DLE was not
                                    bpe:=bpe+1;                // written before
                                    PortInBuf[bpe]:=c;
                                    bpe:=bpe+1;
                                    escape:=false;
                                  end;
                         DLE: begin
                                PortInBuf[bpe]:=c;      // write only one DLE
                                bpe:=bpe+1;
                                escape:=false;
                              end;
                         else begin
                                escape:=false;          // should not occur
                              end;
                       end; {case}
                     end
                     else    // escape=false
                     begin
                       if (c=chr(DLE)) then
                          escape:=true // do not write to buffer, do not increment bpe
                       else
                       begin                            // this is the normal case
                         PortInBuf[bpe]:=c;
                         bpe:=bpe+1;
                       end;
                     end;
                   end;
               end;
  end; {case}
  Result:=0;
end;

function TForm1.clearbuffer:integer;
begin
  FillChar(PortInBuf,sizeof(PortInBuf),0);    // clear buffer
  bps:=0;                                     // reset indices
  bpe:=0;
  escape:=false;                              // reset escape sequence indicator
  Result:=0;
end;

// sendarray has to be repeated for each datasize, due to the strict typing
// of the objectpascal language

function TForm1.sendarray12(outarray:array12):integer;
// all DLE values (apart from those in the DLE/STX and DLE ETX sequences) have
// to be duplicated
var ret:boolean;
    teller:integer;
begin
  ret:=true;
  for teller:=0 to sizeof(outarray)-1 do
  begin
    ret:=ret and _ComWrite(comport,chr(outarray[teller]));
    snooze(1);
    if (teller>=2) and                          // past DLE/STX
       (teller<=(sizeof(outarray)-3)) and       // before DLE/ETX
       (outarray[teller]=DLE) then
    begin
      ret:=ret and _ComWrite(comport,chr(DLE));   // duplicate DLE
      snooze(1);
    end;
  end;
  if not ret
  then ShowMessage(format('Error %d in _ComWrite',[ret]));
  Result:=0;
end;

function TForm1.sendarray14(outarray:array14):integer;
// all DLE values (apart from those in the DLE/STX and DLE ETX sequences) have
// to be duplicated
var ret:boolean;
    teller:integer;
begin
  ret:=true;
  for teller:=0 to sizeof(outarray)-1 do
  begin
    ret:=ret and _ComWrite(comport,chr(outarray[teller]));
    snooze(1);
    if (teller>=2) and                          // past DLE/STX
       (teller<=(sizeof(outarray)-3)) and       // before DLE/ETX
       (outarray[teller]=DLE) then
    begin
      ret:=ret and _ComWrite(comport,chr(DLE));   // duplicate DLE
      snooze(1);
    end;
  end;
  if not ret
  then ShowMessage(format('Error %d in _ComWrite',[ret]));
  Result:=0;
end;

function TForm1.sendarray53(outarray:array53):integer;
// This function is used to send the data part of the measurement configuration
// all DLE values (apart from those in the DLE/STX and DLE ETX sequences) have
// to be duplicated
var ret:boolean;
    teller:integer;
begin
  ret:=true;
  for teller:=0 to sizeof(outarray)-1 do
  begin
    ret:=ret and _ComWrite(comport,chr(outarray[teller]));
    snooze(1);
    if (teller>=2) and                          // past DLE/STX
       (teller<=(sizeof(outarray)-3)) and       // before DLE/ETX
       (outarray[teller]=DLE) then
    begin
      ret:=ret and _ComWrite(comport,chr(DLE));   // duplicate DLE
      snooze(1);
    end;
  end;
  if not ret
  then ShowMessage(format('Error %d in _ComWrite',[ret]));
  Result:=0;
end;

procedure TForm1.open_comm_btnClick(Sender: TObject);
var bufout,bufout2:array12;
    time1,time2:cardinal;
    AVS_Id:AvsIdentityType;
    teller:integer;
    p_id:^AvsIdentityType;
    devcon:DeviceConfigType;
    p_devcon:pDeviceConfigType;
begin
  if (_ComPortPresent(comport)<>1)
  then ShowMessage('COM1 not present');
  _ComInit(comport);
  if not _ComValid(comport)
  then ShowMessage('Error opening COM1');
  _ComSetState(comport,PortBaud,PortDataBits,PortStopBits, PortParity, PortHandShaking);
  _ComSetBufSize(comport,65536,256);    // use large inputbuffer, as there is no handshaking!
  _ComGetSetEventProc(comport,@EventProc);
  _ComBufClear(comport,DIR_INC);        // clear SuperCom buffers
  _ComBufClear(comport,DIR_OUT);

  clearbuffer;

  bufout[00]:=DLE;
  bufout[01]:=STX;     
  bufout[02]:=$20;
  bufout[03]:=$00;
  bufout[04]:=$01;     // node address
  bufout[05]:=$00;     // reserved
  bufout[06]:=$02;     // length of the command
  bufout[07]:=$00;
  bufout[08]:=$13;     // get_ident
  bufout[09]:=$00;
  bufout[10]:=DLE;
  bufout[11]:=ETX;     

  sendarray12(bufout);

  time1:=_ComGetSysTime;
  while ( ((bpe-bps)<sizeof(AVS_Id)) and
          ((cardinal(_ComGetSysTime)-time1)<time_out) ) do
    snooze(1);
  if ((bpe-bps)>=sizeof(AVS_Id)) then     // AVS_Id is 97 bytes in size
  begin
    p_id:=@PortInBuf;
    AVS_Id:=p_id^;   // the contents of PortInBuf are now copied over AVS_Id
    // this does NOT write past AVS_Id !!
    Edit2.Text:=copy(AVS_Id.SerialNumber,1,9);
  end;

  clearbuffer;

  bufout2[00]:=DLE;
  bufout2[01]:=STX;
  bufout2[02]:=$20;
  bufout2[03]:=$00;
  bufout2[04]:=$01;     // node address
  bufout2[05]:=$00;     // reserved
  bufout2[06]:=$02;     // length of the command
  bufout2[07]:=$00;
  bufout2[08]:=$01;     // get_device_configuration
  bufout2[09]:=$00;
  bufout2[10]:=DLE;
  bufout2[11]:=ETX;     

  sendarray12(bufout2);

  time2:=_ComGetSysTime;
  while ( ((bpe-bps)<sizeof(devcon)) and
          ((cardinal(_ComGetSysTime)-time2)<time_out) ) do
    snooze(1);                           // this takes 6-7 seconds at 115200 bps !!
  if ((bpe-bps)>=sizeof(devcon)) then    // devcon is 63496 bytes in size
  begin
    p_devcon:=@PortInBuf;
    devcon:=p_devcon^;    // the contents of PortInBuf are now copied over devcon
    for teller:=0 to NR_WAVELEN_POL_COEF-1 do
       Form1.fit[teller]:=swapsingle(devcon.m_Detector.m_aFit[teller]);
  end;
  SingleScanBtn.Enabled:=true;
end;

function TForm1.readinputvalues:boolean;
var OK:boolean;
    fout1,fout2,fout3:integer;
begin
  val(EdtAvg.Text,Avg,fout1);
  EdtIntTime.Text:=StringReplace(EdtIntTime.Text,',','.',[rfReplaceAll]);
  val(EdtIntTime.Text,IntTime,fout2);
  val(EdtNrms.Text,Nrms,fout3);
  OK:=(fout1=0) and (fout2=0) and (fout3=0) and (Nrms>0);
  if not OK
  then ShowMessage('Input Error');
  Result:=OK;
end;

// Please note the swap functions that reverse the byte order of all data,
// both incoming and outgoing. The AS-5216 uses a ColdFire microprocessor,
// which is of Motorola heritage and therefore big-endian. PC's are of Intel
// heritage and therefore little-endian.
// This means that the byte order of all 16-bit and 32-bit values has to be
// reversed.

function TForm1.swapsingle(value_in:single):single;
// a variant record lets you treat the same data as belonging to different types
// this allows you to swap the 4 bytes and access the result as a single again
type s = record
           case byte of
            0: (sa:array[1..4] of byte);
            1: (res:single);
         end;
var inval,outval:s;
    teller:byte;
begin
  inval.res:=value_in;
  for teller:=1 to 4 do outval.sa[teller]:=inval.sa[5-teller];
  Result:=outval.res;
end;

function TForm1.swapint32(value_in:integer):integer;
// a variant record lets you treat the same data as belonging to different types
// this allows you to swap the 4 bytes and access the result as an int32 again
type s = record
           case byte of
            0: (sa:array[1..4] of byte);
            1: (res:integer);
         end;
var inval,outval:s;
    teller:byte;
begin
  inval.res:=value_in;
  for teller:=1 to 4 do outval.sa[teller]:=inval.sa[5-teller];
  Result:=outval.res;
end;

function TForm1.getlambda(pixel:integer):single;
// This is the polynomial that is used to calculate wavelength values
// from pixel values. The coefficients are stored in the eeprom of the
// AS-5216 and extracted with the get_device_configuration command (above)
begin
  Result:=fit[0]+
          fit[1]*pixel*1.0+
          fit[2]*pixel*pixel*1.0+
          fit[3]*pixel*pixel*pixel*1.0+
          fit[4]*pixel*pixel*pixel*pixel*1.0;
end;

procedure TForm1.SingleScanBtnClick(Sender: TObject);
type // sony ILX: 18 dead pixels
     //           2048 datapixels
     //           single average -> 1 word per pixel
     //           multiple average -> 1 DWORD per pixel
     sony_single_measdatatype=record
                                prefix: array[0..9] of byte;
                                timestamp:cardinal;
                                deadpix: array[0..deadpix-1] of word;
                                pixels: array[startpix..stoppix] of word;
                                postfix: array[0..1] of byte;   // DLE/ETX
                              end;
     sony_multi_measdatatype=record
                               prefix: array[0..9] of byte;
                               timestamp:cardinal;
                               averages:word;
                               deadpix: array[0..deadpix-1] of DWORD;
                               pixels: array[startpix..stoppix] of DWORD;
                               postfix: array[0..1] of byte;   // DLE/ETX
                              end;
var teller,measnr,response_size:integer;
    sum_of_counts,time1,time2,time3:cardinal;
    bufout:array53;
    bufout2:array14;
    bufout3:array12;
    measconfig:SendMeasConfigType;
    p_bufout:^array53;
    sony_single_meas:sony_single_measdatatype;
    p_sony_single_meas:^sony_single_measdatatype;
    sony_multi_meas:sony_multi_measdatatype;
    p_sony_multi_meas:^sony_multi_measdatatype;
begin
  if readinputvalues then
  begin
    clearbuffer;
    // Remember to swap all 16 and 32 bit values (both integer and float !!)
    // Only the values used are swapped here, zero values do not change when
    // swapped, therefore they do not need to be swapped here !!
    measconfig.prefix[00]:=DLE;
    measconfig.prefix[01]:=STX;
    measconfig.prefix[02]:=$20;
    measconfig.prefix[03]:=$00;
    measconfig.prefix[04]:=$01;     // node address
    measconfig.prefix[05]:=$00;     // reserved
    measconfig.prefix[06]:=$2B;     // length of the command
    measconfig.prefix[07]:=$00;
    measconfig.prefix[08]:=$05;     // prepare_measurement
    measconfig.prefix[09]:=$00;
    measconfig.m_Meas.m_StartPixel:=swap(startpix);
    measconfig.m_Meas.m_StopPixel:=swap(stoppix);  // the ILX has 2048 pixels, #0 to #2047
    measconfig.m_Meas.m_IntegrationTime:=swapsingle(IntTime);
    measconfig.m_Meas.m_IntegrationDelay:=0;
    measconfig.m_Meas.m_NrAverages:=swapint32(Avg);
    measconfig.m_Meas.m_CorDynDark.m_Enable:=0;
    measconfig.m_Meas.m_CorDynDark.m_ForgetPercentage:=0;
    measconfig.m_Meas.m_Smoothing.m_SmoothPix:=0;
    measconfig.m_Meas.m_Smoothing.m_SmoothModel:=0;
    measconfig.m_Meas.m_SaturationDetection:=0;
    measconfig.m_Meas.m_Trigger.m_Mode:=0;
    measconfig.m_Meas.m_Trigger.m_Source:=0;
    measconfig.m_Meas.m_Trigger.m_SourceType:=0;
    measconfig.m_Meas.m_Control.m_StrobeControl:=0;
    measconfig.m_Meas.m_Control.m_LaserDelay:=0;
    measconfig.m_Meas.m_Control.m_LaserWidth:=0;
    measconfig.m_Meas.m_Control.m_LaserWaveLength:=0;
    measconfig.m_Meas.m_Control.m_StoreToRam:=0;
    measconfig.postfix[0]:=DLE;
    measconfig.postfix[1]:=ETX;
    p_bufout:=@measconfig;
    bufout:=p_bufout^;

    sendarray53(bufout);

    time1:=_ComGetSysTime;      // response of 12 bytes
    while ( ((bpe-bps)<12) and
            ((cardinal(_ComGetSysTime)-time1)<time_out) ) do
      snooze(1);
    if ((bpe-bps)<>12) or (ord(PortInBuf[bps+8])<>$85)
    then ShowMessage('Error in Prepare_Measurement');

    clearbuffer;

    bufout2[00]:=DLE;
    bufout2[01]:=STX;
    bufout2[02]:=$20;
    bufout2[03]:=$00;
    bufout2[04]:=$01;       // node address
    bufout2[05]:=$00;       // reserved
    bufout2[06]:=$04;       // length of the command
    bufout2[07]:=$00;
    bufout2[08]:=$06;       // start_measurement
    bufout2[09]:=$00;
    bufout2[10]:=Hi(Nrms);  // high byte of the number of measurements
    bufout2[11]:=Lo(Nrms);  // low byte of the number of measurements
    // this is in reversed order too, as it goes out to the Coldfire  
    bufout2[12]:=DLE;
    bufout2[13]:=ETX;       

    sendarray14(bufout2);

    if Avg<=1  // size of response depends on # of averages
    then response_size:=sizeof(sony_single_measdatatype)
    else response_size:=sizeof(sony_multi_measdatatype);

    time2:=_ComGetSysTime;
    while ( ((bpe-bps)<12) and  // response of 12 bytes to Start_Measurement
            ((cardinal(_ComGetSysTime)-time2)<time_out) ) do
      snooze(1);
    if ((bpe-bps)<12) or (ord(PortInBuf[bps+8])<>$86)
    then ShowMessage('Error in Start_Measurement')
    else bps:=bps+12;

    // This is the only time PortInBuf contains 2 frames:
    //  1) the 12 byte answer to the Start_Measurement command
    //  2) the first measurement (much larger)
    // By incrementing the starting index, we can make sure we read the
    // second frame. This index is reset again in the next clearbuffer.

    measnr:=0;
    while measnr<Nrms do
    begin
      time3:=_ComGetSysTime;
      while ( ((bpe-bps)<response_size) and
              ((cardinal(_ComGetSysTime)-time3)<time_out) ) do
        snooze(1);
      if ((bpe-bps)>=response_size) then
      begin
        case ord(PortInBuf[bps+8]) of
          $B0: begin  // number of averages = 1
                 p_sony_single_meas:=@PortInBuf[bps];
                 sony_single_meas:=p_sony_single_meas^;
                 Edit1.Text:=format('%d',[swapint32(sony_single_meas.timestamp)]);
                 Series1.Clear;
                 for teller := startpix to stoppix do
                   Series1.AddXY(getlambda(teller),swap(sony_single_meas.pixels[teller]),'',TColor(clBlack));
               end;
          $B1: begin  // number of averages > 1
                 p_sony_multi_meas:=@PortInBuf[bps];
                 sony_multi_meas:=p_sony_multi_meas^;
                 Edit1.Text:=format('%d',[swapint32(sony_multi_meas.timestamp)]);
                 if swap(sony_multi_meas.averages)<>Avg then
                   ShowMessage('Error in Number of Averages');
                 Series1.Clear;
                 for teller := startpix to stoppix do
                 begin
                   sum_of_counts:=swapint32(sony_multi_meas.pixels[teller]);
                   Series1.AddXY(getlambda(teller),1.0*sum_of_counts/Avg,'',TColor(clBlack));
                 end;
               end;
        end; {case}
      end;
      measnr:=measnr+1;
      if measnr<Nrms then
      begin
        Snooze(1); // give Windows some air to redraw the Chart
        clearbuffer;

        // send acknowledgement
        bufout3[00]:=DLE;
        bufout3[01]:=STX;
        bufout3[02]:=$21;
        bufout3[03]:=$00;
        bufout3[04]:=$01;       // node address
        bufout3[05]:=$00;       // reserved
        bufout3[06]:=$02;       // length of the command
        bufout3[07]:=$00;
        bufout3[08]:=$C0;       // acknowledge
        bufout3[09]:=$00;
        bufout3[10]:=DLE;
        bufout3[11]:=ETX;

        sendarray12(bufout3);
      end;
    end;
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
// setting the focus in TForm1.FormCreate is too early ! 
begin
  open_comm_btn.SetFocus;
end;

procedure TForm1.close_comm_btnClick(Sender: TObject);
begin
  SingleScanBtn.enabled:=false;
  _ComReset(comport);
end;

end.
